// Frequencies of alphabetic characters in a text
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>

#define LETTERS 26
#define CHUNK 256
unsigned counts[LETTERS];
char s[CHUNK+1];

char *filename = NULL; // will point to filename command line argument, if any
char outcase = 'A';   // change to 'a' for lowercase output
int minoutcount = 0;  // minimum frequency for output

void printUsage(void) {
  printf("charfreq [options] [filename]\n");
  printf("filename      input text (default: stdin)\n");
  printf("Options:\n");
  printf("-t n          frequency threshold\n");
  printf("-l            lowercase output\n");
  printf("-h            this help\n");
}

void illegalopt(const char *o) {
      fprintf(stderr, "illegal option: %s\n", o);
      printUsage();
      exit(EXIT_FAILURE);
}

void parsecmdln(int n, char *args[]) {
  int i = 0;

  while (++i < n) {
    char *p = NULL;
    long th;

    if (args[i][0] != '-') {
      filename = args[i];     // must be filename
      break;                  // ignore anything following
    }

    switch (args[i][1]) {
      case 'l':
        outcase = 'a';
        break;

      case 't':
        th = strtol(args[++i], &p, 10); // numeric argument follows
        if (p == args[i] || th < 0) {
          fprintf(stderr, "invalid or negative threshold\n");
          exit(EXIT_FAILURE);
        }
        minoutcount = th;
        break;

      case 'h':
        printUsage();
        exit(0);
        break;

      default:
        illegalopt(args[i]);
    }
  }
}

int main(int argc, char *argv[]) {
  int i;

  parsecmdln(argc, argv);

  while ( fgets(s, sizeof(s), stdin) != NULL ) {
    char c, *p = s;

    while ((c = *p++))
      if (isalpha(c))
        ++counts[toupper(c) - 'A'];
  }

  for(i=0; i<LETTERS; ++i)
    if (counts[i] >= minoutcount)
      printf("%c\t%9u\n", i + outcase, counts[i]);

  return 0;
}
